import copy
import networkx as nx
import sys
import csv
import os
import Queue
import random
import time
import thread
import matplotlib.pyplot as plt
import matplotlib.patches as patches
from multiprocessing import Pool

import numpy
from collections import defaultdict
import math
import shutil
import tempfile

import subprocess
import community

sys.path.append('../epidemicSpread/lineGraph')



    

def doInfomapPartitioning(Network, filename):

    print "Doing infomap partitioning"

    pajekFile = filename + ".net"
    infomapFile = filename + ".tree"


    for node in Network.nodes():
        Network.node[node]['partition'] = None
        print "node: ", node

        
    print "Writing pajekFile: ", pajekFile
    f = open(pajekFile, 'w')
    
    count = 1
    f.write("*Vertices " + str(len(G.nodes())) + "\n")

    nodesToIDMap = dict()
    for node in Network.nodes():
        f.write(str(count) + " " + str(node) + "\n")
        nodesToIDMap[node]=count
        count += 1


    f.write("*Edges " + str(len(G.edges())) + "\n")
    for src,dst in Network.edges():
        f.write(str( nodesToIDMap[src]) + " " + str(nodesToIDMap[dst]) + " " + str(int(Network[src][dst]['weight'])) + "\n")

    f.close() 
    print "Done writing pajekFile: ", pajekFile

    os.system("infomap_undir/infomap 0 " + pajekFile + " 10")


    #read back in infomap assignments
    partitionFile = csv.reader(open(infomapFile), delimiter=" ")
    
    print "Reading back in infomap partition file: ", infomapFile

    

    #nodesToPartition = dict()
    for line in partitionFile:
        if not line[0].startswith("#"):
            partitionAssignment = int(line[0].split(":")[0])
            theNode = line[2]
    #        partitionAssignment = int(line[0])
#            print "Got this node from file: ", line[2]
#            theNode = theNode.strip("\"")
#            theNode = theNode.strip()
#            theNode = "\"" + theNode + "\""
#            print "node: ", str(theNode), " partition: ", str(partitionAssignment)
            Network.node[unicode(theNode)]['partition'] = partitionAssignment


#        Network.node[relabelMap_externalToInternal[int(line[1])]]['partition'] = partitionAssignment




def doBlondelPartitioning(Network, filename):
    partition = community.best_partition(Network)
    #dendo = community.generate_dendogram(Network, None)
    #partition = community.partition_at_level(dendo, 0)
    for node in Network:
        Network.node[node]['partition'] = partition[node]
    



def calculateONMI(filename1,filename2):
    proc = subprocess.Popen(["../temporalCommunity/NMI/onmi", filename1,filename2],stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    (out,err)=proc.communicate()

    return out.split("\n")[-2].strip()

def calculateNMIFortunato(filename1,filename2):
    proc = subprocess.Popen(["../temporalCommunity/mutual3/mutual", filename1,filename2],stdout=subprocess.PIPE,stderr=subprocess.PIPE)
    (out,err)=proc.communicate()

    return out.split("\t")[1].strip()





colors = ["#ff6347","#4169e1","#98fb98","#ffd700"]
shapes = ["pentagon","diamond","circle","triangle"]


def convertWeightToColor(weight):

    hue = math.log(1.0+ 10.0 * weight)
    hue = hue / 19.0 #scaling factor.
    hue = min(hue,1.0)
    hue = max(hue,0.3)
    hue = 1.0 - hue

    return "" +( "%.2f" % hue) + " 1.0 1.0"



def writeOutGraphviz(G, outputFileName = "renderPlain"):

    outputGraph = copy.deepcopy(G)

    maxLength = 0.0
    for src,dst in outputGraph.edges():
        if (G[src][dst]['weight'] + G[dst][src]['weight']) > maxLength:
            maxLength = (G[src][dst]['weight'] + G[dst][src]['weight'])


    



    for src,dst in outputGraph.edges():
        if (G[src][dst]['weight'] + G[dst][src]['weight']) != 0:
            outputGraph[src][dst]['len'] = 10 * maxLength / (G[src][dst]['weight'] + G[dst][src]['weight'])
    #        outputGraph[src][dst]['weight'] = (G[src][dst]['weight'] + G[dst][src]['weight'])
            outputGraph[src][dst]['penwidth'] = float(outputGraph[src][dst]['weight']) / 50000  #math.log(1.0 + outputGraph[src][dst]['weight'])
        else:
            outputGraph[src][dst]['len'] = 1000000 #TODO hack, if go further with this, should instead remove this edge that has 0 weight from the output graph.
            

    for node in outputGraph.nodes():
        outputGraph.node[node]['label'] = node
        outputGraph.node[node]['color'] = colors[outputGraph.node[node]['groundTruth'] % len(colors) ]
        outputGraph.node[node]['shape'] = shapes[outputGraph.node[node]['groundTruth'] % len(shapes) ]
        

    #get the weight of the lower 50th percentigle
    lengths = []
    for src,dst in outputGraph.edges():
#        lengths.append(outputGraph[src][dst]['weight'])
        lengths.append(outputGraph[src][dst]['len'])
    
    sortedLengths = sorted(lengths)
    
    for src,dst in outputGraph.edges():
        
        if outputGraph[src][dst]['len'] > sortedLengths[len(sortedLengths)/2]:
        #if outputGraph[src][dst]['weight'] < sortedLengths[len(sortedLengths)/2]:
            outputGraph[src][dst]['style'] = "invis"

    nx.write_dot(outputGraph, outputFileName + "_graphviz.dot")

    #os.system('neato -Gsize="100000,100000" -Goverlap="expand" -Gratio="1.0" -Goutputorder="edgesfirst" -Nstyle="filled" -Nfixedsize="True" -Nfontsize=80 -Nfontcolor="#eeeeee" -GK=10000000 -Nwidth=7 -Tpdf ' + outputFileName + "_graphviz.dot" +  " -o " + outputFileName + "_graphviz.pdf")
    #os.system('neato -Goverlap="expand" -Gratio="1.0" -Goutputorder="edgesfirst" -Nregular="true" -Nwidth=4 -Nstyle="filled" -Ecolor="#aaaaaa44" -Tpdf ' + outputFileName + "_graphviz.dot" +  " -o " + outputFileName + "_graphviz.pdf")

#    os.system('neato -Gstart=10 -Goverlap="expand" -Gratio="1.0" -Goutputorder="edgesfirst" -Nregular="true" -Nwidth=4 -Nstyle="filled" -Ecolor="#aaaaaa" -Tpdf ' + outputFileName + "_graphviz.dot" +  " -o " + outputFileName + "_graphviz.pdf")
    os.system('neato -Gstart=0 -Goverlap="expand" -Gratio="1.0" -Goutputorder="edgesfirst" -Nregular="true" -Nwidth=4 -Nstyle="filled" -Ecolor="#aaaaaa" -Tpdf ' + outputFileName + "_graphviz.dot" +  " -o " + outputFileName + "_graphviz.pdf")



#    for src,dst in outputGraph.edges():
#        outputGraph[src][dst]['weight'] = (G[src][dst]['weight'] + G[dst][src]['weight'])
    #do fructherman reingold
#    plt.clf()

#    pos = nx.spring_layout(outputGraph,dim=2,pos=None,fixed=None,iterations=500,weight='weight',scale=10)
#    colorList = []
#    for node in outputGraph.nodes():    
#        colorList.append(colors[outputGraph.node[node]['groundTruth'] % len(colors) ])
#    nx.draw_networkx_nodes(outputGraph,pos = pos,node_color = colorList)
#    nx.draw_networkx_edges(outputGraph,pos=pos,edge_color='blue')

#    plt.axis('off')
#    plt.axis('tight')
#    plt.savefig("out/" + outputName + ".eps" ,dpi=75)
#    plt.savefig(outputFileName + "_networkx.pdf" ,dpi=75, bbox_inches='tight')


def writeOutClusterAndGroundTruthFiles(G,filename):
    internalToExternalID = dict()
    count = 1
    for node in G.nodes():
        internalToExternalID[node] = count
        count += 1


    f = open(filename + "_foundClustersOriginalNames", 'w')
    clusters = defaultdict(set)

    for node in G.nodes():
        clusters[G.node[node]['partition']].add(node)
    for key in clusters:
        f.write(" ".join([str(x) for x in clusters[key]]) + "\n")
    f.close()

    f = open(filename + "_foundClusters", 'w')
    clusters = defaultdict(set)

    for node in G.nodes():
        clusters[G.node[node]['partition']].add(node)
    for key in clusters:
        f.write(" ".join([str(internalToExternalID[x]) for x in clusters[key]]) + "\n")
    f.close()

    f = open(filename + "_groundTruthClusters", 'w')
    clusters = defaultdict(set)

    for node in G.nodes():
#        print "Node: ", node , " groundTruth: ", G.node[node]['groundTruth']
        if 'groundTruth' in G.node[node]:
            clusters[G.node[node]['groundTruth']].add(node)
        else:
            print >> sys.stderr, "Error, node: ", node , " from network, has no ground truth value!"
    for key in clusters:
        f.write(" ".join([str(internalToExternalID[x]) for x in clusters[key]]) + "\n")
    f.close()
 







G = nx.Graph()

file = csv.reader(open(sys.argv[1],'r'), delimiter=' ',quotechar='"')

for row in file:
    G.add_edge(row[0],row[1])
    if 'weight' in G[row[0]][row[1]]:
        G[row[0]][row[1]]['weight'] += int(row[2])
    else:
        G[row[0]][row[1]]['weight'] = int(row[2])


#G = nx.read_weighted_edgelist(sys.argv[1])
#print "Nodes: ", len( G.nodes())
#print "Edges: ", len( G.edges())

#doInfomapPartitioning(G,sys.argv[1])
doBlondelPartitioning(G,sys.argv[1])
#calculateModularity(G,sys.argv[1])

#for node in G.nodes():
#    print "Node: ", node , " cluster: ", G.node[node]['partition']

groundTruth = csv.reader(open(sys.argv[2]) , delimiter = " ", quotechar='"')
for line in groundTruth:
    theNode = line[0]
    if theNode in G.nodes():
        G.node[theNode]['groundTruth'] = int(line[1])
    else:
        print >>sys.stderr ,"Error, node: ", theNode , " from ground truth, not found in network!"



writeOutGraphviz(G,sys.argv[1])

writeOutClusterAndGroundTruthFiles(G,sys.argv[1])

LFKNmi = calculateNMIFortunato(sys.argv[1] + "_foundClusters", sys.argv[1] + "_groundTruthClusters")

#onmi = calculateONMI(sys.argv[1] + "_foundClusters", sys.argv[1] + "_groundTruthClusters")

print "NMI_vs_groundTruth,",LFKNmi#, ",",onmi
#print "ONMI against ground truth: "


